@@ -1,12 +1,19 @@
 import os
-from django.conf.urls.defaults import *
+from django.contrib import admin
+try:
+    from django.conf.urls import patterns, include
+except ImportError:
+    from django.conf.urls.defaults import patterns, include
 
-urlpatterns = patterns('',
+admin.autodiscover()
+
+
+urlpatterns = patterns(
+    '',
+    (r'^admin/', include(admin.site.urls)),
     (r'^comments/', include('django.contrib.comments.urls')),
-    (r'^accounts/', include('registration.backends.default.urls')),
     (r'^media/(?P<path>.*)$', 'django.views.static.serve',
         {'document_root': os.path.join(os.path.dirname(__file__), 'media')}),
     (r'auth/', include('django.contrib.auth.urls')),
     (r'', include('actstream.urls')),
 )
-
